require "multi_typograf/version"
require "multi_typograf/filter"

module MultiTypograf
  class Error < StandardError
    def initialize(message="", backtrace=nil)
      super(message)
      self.set_backtrace(backtrace) if backtrace
    end
  end

  @adapter = nil

  REQUIREMENT_MAP = [
    ["art_typograf", :art_typograf],
    ["typographer/parsers/basic",  :typographer],
    ["typographica", :typographica],
    ["typograf", :typograf]
  ]

  class << self

    # The default adapter based on what you currently
    # have loaded and installed. First checks to see
    # if any adapters are already loaded, then checks
    # to see which are installed if none are loaded.
    def default_adapter
      return :art_typograf if defined?(::ArtTypograf)
      return :typographer if defined?(::TypographerHelper::Parsers::Basic)
      return :typographica if defined?(::Typographica)
      return :typograf if defined?(::Typograf)

      REQUIREMENT_MAP.each do |(library, adapter)|
        begin
          require library
          return adapter
        rescue LoadError
          next
        end
      end

      :typograf
    end
    # :nodoc:
    alias :default_engine :default_adapter

    # Get the current adapter class.
    def adapter
      return @adapter if @adapter
      self.use self.default_adapter
      @adapter
    end
    # :nodoc:
    alias :engine :adapter

    # Set the adapter utilizing a symbol, string, or class.
    # Supported by default are:
    #
    # * <tt>:typograf</tt>
    # * <tt>:art_typograf</tt>
    # * <tt>:typographer</tt>
    # * <tt>:typography</tt>
    # * <tt>:typographica</tt>
    def use(new_adapter)
      @adapter = load_adapter(new_adapter)
    end
    alias :adapter= :use
    # :nodoc:
    alias :engine= :use

    def load_adapter(new_adapter)
      case new_adapter
      when String, Symbol
        require "multi_typograf/adapters/#{new_adapter}"
        self::Adapters.const_get(:"#{new_adapter.to_s.split('_').map{|s| s.capitalize}.join('')}")
      when NilClass, FalseClass
        default_adapter = self.default_adapter
        require "multi_typograf/adapters/#{default_adapter}"
        self::Adapters.const_get(:"#{default_adapter.to_s.split('_').map{|s| s.capitalize}.join('')}")
      when Class
        new_adapter
      else
        raise "Did not recognize your adapter specification. Please specify either a symbol or a class."
      end
    end

    def current_adapter(options)
      if new_adapter = (options || {}).delete(:adapter)
        load_adapter(new_adapter)
      else
        adapter
      end
    end

    # Minify CSS
    def process(string, options={})
      string = string.read if string.respond_to?(:read)

      adapter = current_adapter(options)
      if defined?(adapter::ParseError)
        begin
          adapter.process(string, options)
        rescue adapter::ParseError => exception
          raise ::MultiTypograf::Error.new(exception.message, exception.backtrace)
        end
      else
        adapter.process(string, options)
      end
    end

  end
end
